part of easy_church_json;

/// EasyChurchJsonClass
///
/// This class is a factory so it has only one instance.
class EasyChurchJson {
  /// General instance.
  static EasyChurchJson _instance = new EasyChurchJson._internal();

  /// Constructor.
  factory EasyChurchJson() {
    return _instance;
  }

  /// Internal constructor.
  EasyChurchJson._internal();

  ///Get all churches
  ///
  /// Gets a list of churches
  Future<List<SingleChurch>> getAllChurches() async {
    List<SingleChurch> list = [];
    try {
      var response =
          await http.get("https://easy-church-api.herokuapp.com/getChurches");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleChurch singleChurch = SingleChurch.fromJson(j[i]);
          list.add(singleChurch);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get all notifications of a church -> ID
  Future<List<SingleNotification>> getNotifications(String id) async {
    List<SingleNotification> list = [];
    try {
      var response = await http
          .get("https://easy-church-api.herokuapp.com/getNotifications/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleNotification singleNotification =
              SingleNotification.fromJson(j[i]);
          list.add(singleNotification);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get all notifications of a church -> ID
  Future<List<SingleAnnouncement>> getAnnouncements(String id) async {
    List<SingleAnnouncement> list = [];
    try {
      var response = await http
          .get("https://easy-church-api.herokuapp.com/getAnnouncements/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleAnnouncement singleAnnouncement =
              SingleAnnouncement.fromJson(j[i]);
          list.add(singleAnnouncement);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get all events of a church -> ID
  Future<List<SingleEvent>> getEvents(String id) async {
    List<SingleEvent> list = [];
    try {
      var response =
          await http.get("https://easy-church-api.herokuapp.com/getEvents/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleEvent singleEvent = SingleEvent.fromJson(j[i]);
          list.add(singleEvent);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Subscribe to a church
  ///
  /// Requires an Id of a church and the phones IMEI number
  Future<String> followChurch(String churchId, String IMEI) async {
    try {
      var response = await http.get(
          "https://easy-church-api.herokuapp.com/subChurch/$churchId/$IMEI");

      if (response.statusCode == 200) {
        String resp = response.body;

        if (resp == "Save successful") {
          //User has been successfully subscribed
          return "Success";
        } else {
          //Failed
          return "Failed";
        }
      } else {
        return "Network Fail";
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get a church which the user is subscribed to
  ///
  /// Requires the phones IMEI number
  Future<List<SingleChurch>> getChurch(String IMEI) async {
    List<SingleChurch> list = [];

    try {
      var response = await http
          .get("https://easy-church-api.herokuapp.com/getChurch/$IMEI");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleChurch singleChurch = SingleChurch.fromJson(j[i]);
          list.add(singleChurch);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get all media of a church -> ID
  Future<List<SingleMedia>> getMediaForChurch(String id) async {
    List<SingleMedia> list = [];
    try {
      var response =
          await http.get("https://easy-church-api.herokuapp.com/getMedia/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleMedia singleMedia = SingleMedia.fromJson(j[i]);
          list.add(singleMedia);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get all media irrespective of churches
  Future<List<SingleMedia>> getMedia() async {
    List<SingleMedia> list = [];
    try {
      var response =
          await http.get("https://easy-church-api.herokuapp.com/getMedias");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleMedia singleMedia = SingleMedia.fromJson(j[i]);
          list.add(singleMedia);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get all media irrespective of churches
  Future<void> downloadMedia(String id) async {
    List<SingleMedia> list = [];
    try {
      Response response = await Dio()
          .get("https://easy-church-api.herokuapp.com/downloadMedia/$id");
      print(response);
    } catch (e) {
      print(e.toString());
    }
  }

//Get the daily verse for a church
  Future<SingleVerse> getDailyVerseForChurch(String id) async {
    try {
      var response = await http
          .get("https://easy-church-api.herokuapp.com/getVerseForChurch/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleVerse singleVerse = SingleVerse.fromJson(j[i]);
          return singleVerse;
        }
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get verse by Id, get verse at index one from returned item
  Future<List<SingleVerse>> getVerseById(String id) async {
    List<SingleVerse> list = [];
    try {
      var response = await http
          .get("https://easy-church-api.herokuapp.com/getVerseById/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleVerse singleVerse = SingleVerse.fromJson(j[i]);
          list.add(singleVerse);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get history verses from church -> ChurchId
  Future<List<SingleVerse>> getVersesFromChurch(String id) async {
    List<SingleVerse> list = [];
    try {
      var response = await http
          .get("https://easy-church-api.herokuapp.com/getVersesForChurch/$id");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleVerse singleVerse = SingleVerse.fromJson(j[i]);
          list.add(singleVerse);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  ///Get history verses
  Future<List<SingleVerse>> getAllVerses() async {
    List<SingleVerse> list = [];
    try {
      var response =
          await http.get("https://easy-church-api.herokuapp.com/getAllVerses");

      if (response.statusCode == 200) {
        var j = json.decode(response.body);
        for (int i = 0; i < j.length; i++) {
          SingleVerse singleVerse = SingleVerse.fromJson(j[i]);
          list.add(singleVerse);
        }

        return list;
      } else {
        print("Failed to get Info");
        return null;
      }
    } on http.ClientException catch (e) {
      print(e.toString());
      return null;
    }
  }

  //Send prayer
  Future<HttpClientResponse> postPrayer(String churchId, String details,
      String user_name, String user_imei, String phone_number) async {
    Map<String, dynamic> body = {
      'details': details,
      'user_name': user_name,
      'user_imei': user_imei,
      'phone_number': phone_number
    };

    try {
      Response response = await Dio().post(
          'https://easy-church-api.herokuapp.com/new/prayer/$churchId',
          data: body,
          options: new Options(
              contentType:
                  ContentType.parse("application/x-www-form-urlencoded")));
      print(response);
    } catch (e) {
      print(e.toString());
    }
  }
}
